# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import os
import shutil
from win_lin import running_on_linux
if running_on_linux()==False:
	import _winreg

phys_path=None
light_dll_path=None
exe_command=None
share_path=None
device_lib_path=None
bin_path=None
lib_path=None

def cal_share_path():
	global share_path
	if running_on_linux()==True:
		if os.path.isfile("main.c")==True:
			share_path=os.getcwd()
		else:
			share_path="/usr/share/opvdm/"
	else:
		try:
			registry_key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, "Software\\opvdm", 0, _winreg.KEY_READ)
			value, regtype = _winreg.QueryValueEx(registry_key, "installpath")
			_winreg.CloseKey(registry_key)
			print "Install path at", value
			share_path=value
		except WindowsError:
			print "No registry key found using default"
			share_path="c:\\opvdm"

def cal_lib_path():
	global lib_path
	if running_on_linux()==True:
		if os.path.isfile("main.c")==True:
			lib_path=os.getcwd()
		else:
			lib_path="/usr/lib64/opvdm/"
	else:
		try:
			registry_key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, "Software\\opvdm", 0, _winreg.KEY_READ)
			value, regtype = _winreg.QueryValueEx(registry_key, "installpath")
			_winreg.CloseKey(registry_key)
			print "Lib path at", value
			lib_path=value
		except WindowsError:
			print "No registry key found using default"
			lib_path="c:\\opvdm"

def cal_bin_path():
	global bin_path
	if running_on_linux()==True:
			bin_path="/bin/"
	else:
		try:
			registry_key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, "Software\\opvdm", 0, _winreg.KEY_READ)
			value, regtype = _winreg.QueryValueEx(registry_key, "installpath")
			_winreg.CloseKey(registry_key)
			print "Lib path at", value
			bin_path=value
		except WindowsError:
			print "No registry key found using default"
			bin_path="c:\\opvdm"

def calculate_paths():
	global share_path
	global lib_path
	global phys_path
	global exe_command
	global device_lib_path
	global light_dll_path

	cal_share_path()
	cal_lib_path()
	cal_bin_path()

	device_lib_path=os.path.join(share_path,"device_lib")
	phys_path=os.path.join(share_path,"phys")

	if running_on_linux() == True:
		if os.path.isfile("./main.c")==True:
			exe_command=os.path.join(os.getcwd(), "go.o")
		else:
			exe_command="opvdm_core"

	else:
		if os.path.isfile("opvdm_core.exe")==True:
			exe_command=os.path.join(os.getcwd(), "opvdm_core.exe")
		else:
			exe_command=os.path.join(share_path,"opvdm_core.exe")


	light_dll_path=os.path.join(lib_path,"light")


def get_share_path():
	global share_path
	return share_path

def get_phys_path():
	global phys_path
	return phys_path

def get_device_lib_path():
	global device_lib_path
	return device_lib_path

def get_bin_path():
	global bin_path
	return bin_path

def get_light_dll_path():
	global light_dll_path
	return light_dll_path

def get_exe_command():
	global exe_command
	return exe_command

def get_exe_name():
	if running_on_linux() == True:
		if os.path.isfile("./go.o")==True:
			exe_name="go.o"
		elif os.path.isfile("./main.c")==True:
			exe_name="go.o"
		else:
			exe_name="opvdm_core"
		return exe_name
	else:
		exe_name="opvdm_core.exe"
		return exe_name

def get_inp_file_path():
	global share_path
	return share_path

def get_icon_file_path():
	global share_path
	return os.path.join(share_path,"gui")

def find_data_file(name):
	global share_path
	return os.path.join(share_path,name)

